#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

const int pwmMotorA = D1;  // Left motor
const int pwmMotorB = D2;  // Right motor
const int dirMotorA = D3;
const int dirMotorB = D4;

// Default motor speed for both motors
int motorSpeed = 500;
int motorSpeedA = 0;
int motorSpeedB = 0;
bool dirA = LOW;
bool dirB = LOW;

// Wi-Fi credentials for AP mode
const char* ssid = "RoboCar";
const char* password = "12345678";  // Min 8 characters

// Create web server
ESP8266WebServer server(80);

// HTML for the web page with joystick control
const char htmlPage[] PROGMEM = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
  <title>Robot Joystick Control</title>
  <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
  <style>
    * {
      box-sizing: border-box;
      font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;
      touch-action: manipulation;
    }
    body {
      margin: 0;
      padding: 0;
      background: linear-gradient(135deg, #1a202c 0%, #2d3748 100%);
      color: #fff;
      min-height: 100vh;
      display: flex;
      flex-direction: column;
      justify-content: center;
      align-items: center;
      overflow: hidden;
    }
    .container {
      width: 95%;
      max-width: 500px;
      background-color: rgba(255, 255, 255, 0.1);
      backdrop-filter: blur(10px);
      border-radius: 20px;
      padding: 25px;
      box-shadow: 0 8px 32px rgba(0, 0, 0, 0.2);
      border: 1px solid rgba(255, 255, 255, 0.18);
      margin: 15px;
    }
    h1 {
      font-size: 28px;
      text-align: center;
      margin-top: 0;
      margin-bottom: 20px;
      text-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
      color: #a0aec0;
    }
    .control-panel {
      margin-top: 15px;
    }
    .slider-container {
      margin: 20px 0;
    }
    .speed-display {
      font-size: 36px;
      text-align: center;
      font-weight: bold;
      margin: 10px 0;
      text-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
      color: #63b3ed;
    }
    .speed-label {
      text-align: center;
      font-size: 18px;
      margin-bottom: 10px;
      opacity: 0.9;
      color: #e2e8f0;
    }
    .slider {
      -webkit-appearance: none;
      width: 100%;
      height: 20px;
      border-radius: 10px;
      background: rgba(255, 255, 255, 0.2);
      outline: none;
      margin: 15px 0;
    }
    .slider::-webkit-slider-thumb {
      -webkit-appearance: none;
      appearance: none;
      width: 30px;
      height: 30px;
      border-radius: 50%;
      background: #4fd1c5;
      cursor: pointer;
      box-shadow: 0 2px 6px rgba(0, 0, 0, 0.2);
    }
    .slider::-moz-range-thumb {
      width: 30px;
      height: 30px;
      border-radius: 50%;
      background: #4fd1c5;
      cursor: pointer;
      box-shadow: 0 2px 6px rgba(0, 0, 0, 0.2);
      border: none;
    }
    .joystick-container {
      position: relative;
      width: 280px;
      height: 280px;
      background-color: rgba(26, 32, 44, 0.7);
      border: 2px solid rgba(102, 126, 234, 0.5);
      border-radius: 50%;
      margin: 0 auto;
      overflow: hidden;
      touch-action: none;
    }
    .joystick-base {
      position: absolute;
      top: 50%;
      left: 50%;
      transform: translate(-50%, -50%);
      width: 80px;
      height: 80px;
      background: radial-gradient(circle, #4fd1c5 0%, #38b2ac 100%);
      border-radius: 50%;
      cursor: pointer;
      box-shadow: 0 4px 8px rgba(0, 0, 0, 0.3);
      user-select: none;
      z-index: 2;
      display: flex;
      justify-content: center;
      align-items: center;
    }
    .joystick-crosshair {
      position: absolute;
      top: 0;
      left: 0;
      right: 0;
      bottom: 0;
      pointer-events: none;
    }
    .joystick-crosshair::before,
    .joystick-crosshair::after {
      content: "";
      position: absolute;
      background-color: rgba(255, 255, 255, 0.2);
      z-index: 1;
    }
    .joystick-crosshair::before {
      width: 2px;
      height: 100%;
      left: 50%;
      transform: translateX(-50%);
    }
    .joystick-crosshair::after {
      width: 100%;
      height: 2px;
      top: 50%;
      transform: translateY(-50%);
    }
    .controls-container {
      margin-top: 20px;
      display: flex;
      justify-content: space-between;
    }
    .button-grid {
      display: grid;
      grid-template-columns: repeat(3, 1fr);
      gap: 10px;
      margin: 20px auto;
      width: 210px;
    }
    .ctrl-button {
      border: none;
      padding: 12px 5px;
      border-radius: 12px;
      font-size: 14px;
      font-weight: bold;
      cursor: pointer;
      transition: all 0.3s ease;
      box-shadow: 0 4px 10px rgba(0, 0, 0, 0.2);
      text-transform: uppercase;
      letter-spacing: 1px;
      height: 50px;
      display: flex;
      justify-content: center;
      align-items: center;
    }
    .forward-btn {
      background-color: #4ade80;
      color: #1e293b;
      grid-column: 2;
      grid-row: 1;
    }
    .left-btn {
      background-color: #a78bfa;
      color: white;
      grid-column: 1;
      grid-row: 2;
    }
    .stop-btn {
      background-color: #f43f5e;
      color: white;
      grid-column: 2;
      grid-row: 2;
    }
    .right-btn {
      background-color: #a78bfa;
      color: white;
      grid-column: 3;
      grid-row: 2;
    }
    .reverse-btn {
      background-color: #3b82f6;
      color: white;
      grid-column: 2;
      grid-row: 3;
    }
    .status {
      text-align: center;
      margin-top: 15px;
      font-size: 16px;
      background-color: rgba(0, 0, 0, 0.3);
      padding: 12px;
      border-radius: 8px;
      display: flex;
      align-items: center;
      justify-content: center;
      color: #a0aec0;
    }
    .status-indicator {
      display: inline-block;
      width: 12px;
      height: 12px;
      border-radius: 50%;
      margin-right: 8px;
    }
    .status-stopped {
      background-color: #f43f5e;
    }
    .status-moving {
      background-color: #4ade80;
    }
    .button-icon {
      font-size: 20px;
      font-weight: bold;
    }
    .control-mode {
      display: flex;
      justify-content: center;
      margin-bottom: 20px;
    }
    .mode-switch {
      background-color: rgba(0, 0, 0, 0.3);
      border-radius: 25px;
      display: inline-flex;
      padding: 4px;
    }
    .mode-option {
      padding: 8px 16px;
      border-radius: 20px;
      cursor: pointer;
      transition: all 0.3s ease;
      font-weight: bold;
    }
    .mode-active {
      background-color: #4fd1c5;
      color: #1a202c;
    }
    .mode-inactive {
      color: #a0aec0;
    }
    .hidden {
      display: none;
    }
  </style>
</head>
<body>
  <div class="container">
    <h1>Robot Control Center</h1>
    
    <div class="control-mode">
      <div class="mode-switch">
        <div class="mode-option mode-active" id="joystick-mode" onclick="switchMode('joystick')">Joystick</div>
        <div class="mode-option mode-inactive" id="buttons-mode" onclick="switchMode('buttons')">Buttons</div>
      </div>
    </div>
    
    <div class="control-panel">
      <div class="slider-container">
        <div class="speed-label">Max Motor Speed</div>
        <div class="speed-display" id="speedDisplay">500</div>
        <input type="range" min="200" max="1023" value="500" class="slider" id="motorSlider" oninput="updateMaxSpeed(this.value)">
      </div>
      
      <div id="joystick-controls">
        <div class="joystick-container" id="joystickArea">
          <div class="joystick-crosshair"></div>
          <div class="joystick-base" id="joystick"></div>
        </div>
      </div>
      
      <div id="button-controls" class="hidden">
        <div class="button-grid">
          <button class="ctrl-button forward-btn" onclick="motorControl('forward')"><span class="button-icon">↑</span></button>
          <button class="ctrl-button left-btn" onclick="motorControl('left')"><span class="button-icon">←</span></button>
          <button class="ctrl-button stop-btn" onclick="motorControl('stop')"><span class="button-icon">■</span></button>
          <button class="ctrl-button right-btn" onclick="motorControl('right')"><span class="button-icon">→</span></button>
          <button class="ctrl-button reverse-btn" onclick="motorControl('reverse')"><span class="button-icon">↓</span></button>
        </div>
      </div>
      
      <div class="status">
        <span class="status-indicator status-stopped" id="statusIndicator"></span>
        <span id="statusText">Motors Stopped</span>
      </div>
    </div>
  </div>
  
  <script>
    // Variables for joystick functionality
    let maxSpeed = 500;
    let joystickActive = false;
    let joystickX = 0;
    let joystickY = 0;
    let joystickInterval = null;
    let lastSentX = 0;
    let lastSentY = 0;
    
    // Get DOM elements
    const joystick = document.getElementById('joystick');
    const joystickArea = document.getElementById('joystickArea');
    const statusIndicator = document.getElementById("statusIndicator");
    const statusText = document.getElementById("statusText");
    
    // Initial position calculation
    const bounds = joystickArea.getBoundingClientRect();
    const centerX = bounds.width / 2;
    const centerY = bounds.height / 2;
    const maxDistance = (bounds.width / 2) - (joystick.clientWidth / 2);
    
    // Initialize joystick listeners
    function initJoystick() {
      // Mouse Events
      joystick.addEventListener('mousedown', startMoveJoystick);
      document.addEventListener('mousemove', moveJoystick);
      document.addEventListener('mouseup', endMoveJoystick);
      
      // Touch Events
      joystick.addEventListener('touchstart', startMoveJoystick);
      document.addEventListener('touchmove', moveJoystick);
      document.addEventListener('touchend', endMoveJoystick);
      document.addEventListener('touchcancel', endMoveJoystick);
    }
    
    // Start joystick movement
    function startMoveJoystick(e) {
      e.preventDefault();
      joystickActive = true;
      
      // Start sending control commands at regular intervals
      if (joystickInterval === null) {
        joystickInterval = setInterval(sendJoystickData, 100);
      }
    }
    
    // Handle joystick movement
    function moveJoystick(e) {
      if (!joystickActive) return;
      e.preventDefault();
      
      const bounds = joystickArea.getBoundingClientRect();
      const centerX = bounds.width / 2;
      const centerY = bounds.height / 2;
      
      // Get current position based on mouse or touch
      let clientX, clientY;
      if (e.type.startsWith('mouse')) {
        clientX = e.clientX;
        clientY = e.clientY;
      } else {
        clientX = e.touches[0].clientX;
        clientY = e.touches[0].clientY;
      }
      
      // Calculate joystick position relative to center
      let deltaX = clientX - (bounds.left + centerX);
      let deltaY = clientY - (bounds.top + centerY);
      
      // Calculate distance from center
      const distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
      
      // Limit to the joystick area
      if (distance > maxDistance) {
        const angle = Math.atan2(deltaY, deltaX);
        deltaX = Math.cos(angle) * maxDistance;
        deltaY = Math.sin(angle) * maxDistance;
      }
      
      // Update joystick position
      joystick.style.transform = `translate(${deltaX}px, ${deltaY}px)`;
      
      // Normalize joystick values to -1 to 1
      joystickX = deltaX / maxDistance;
      joystickY = deltaY / maxDistance;
      
      // Update UI status
      updateStatusUI();
    }
    
    // End joystick movement
    function endMoveJoystick(e) {
      if (!joystickActive) return;
      e.preventDefault();
      joystickActive = false;
      
      // Reset joystick position with animation
      joystick.style.transition = 'transform 0.2s ease-out';
      joystick.style.transform = 'translate(0, 0)';
      setTimeout(() => {
        joystick.style.transition = '';
      }, 200);
      
      // Reset joystick values
      joystickX = 0;
      joystickY = 0;
      
      // Send stop command
      sendMotorControl(0, 0, 0, 0);
      
      // Clear the interval
      if (joystickInterval !== null) {
        clearInterval(joystickInterval);
        joystickInterval = null;
      }
      
      // Update UI status
      updateStatusUI();
    }
    
    // Send joystick data to motor control
    function sendJoystickData() {
      // Avoid sending repeated identical commands to reduce network traffic
      if (joystickX === lastSentX && joystickY === lastSentY) return;
      
      lastSentX = joystickX;
      lastSentY = joystickY;
      
      // Convert joystick values to motor speeds
      // Y-axis: forward/reverse, X-axis: left/right
      const y = -joystickY; // Invert Y so up is positive
      const x = joystickX;
      
      // Calculate left and right motor speeds using differential steering
      let leftSpeed = Math.round(maxSpeed * (y + x));
      let rightSpeed = Math.round(maxSpeed * (y - x));
      
      // Determine motor directions
      const leftDir = leftSpeed >= 0 ? 0 : 1;
      const rightDir = rightSpeed >= 0 ? 0 : 1;
      
      // Use absolute values for PWM
      leftSpeed = Math.abs(leftSpeed);
      rightSpeed = Math.abs(rightSpeed);
      
      // Limit values to valid range
      leftSpeed = Math.min(1023, Math.max(0, leftSpeed));
      rightSpeed = Math.min(1023, Math.max(0, rightSpeed));
      
      // Send control command
      sendMotorControl(leftSpeed, leftDir, rightSpeed, rightDir);
    }
    
    // Update status indicator based on joystick position
    function updateStatusUI() {
      statusIndicator.classList.remove("status-stopped", "status-moving");
      
      if (joystickX === 0 && joystickY === 0) {
        statusIndicator.classList.add("status-stopped");
        statusText.innerText = "Motors Stopped";
      } else {
        statusIndicator.classList.add("status-moving");
        
        // Determine direction for status text
        let direction = "";
        const absX = Math.abs(joystickX);
        const absY = Math.abs(joystickY);
        
        if (absY > absX * 2) {
          // Mostly vertical movement
          direction = joystickY < 0 ? "Forward" : "Backward";
        } else if (absX > absY * 2) {
          // Mostly horizontal movement
          direction = joystickX > 0 ? "Right" : "Left";
        } else if (absX > 0.3 && absY > 0.3) {
          // Diagonal movement
          if (joystickX > 0 && joystickY < 0) direction = "Forward-Right";
          else if (joystickX < 0 && joystickY < 0) direction = "Forward-Left";
          else if (joystickX > 0 && joystickY > 0) direction = "Backward-Right";
          else direction = "Backward-Left";
        } else {
          direction = "Moving";
        }
        
        statusText.innerText = direction;
      }
    }
    
    // Send motor control commands directly (for button interface)
    function motorControl(action) {
      if (action === "forward") {
        sendMotorControl(maxSpeed, 0, maxSpeed, 0);
      } else if (action === "reverse") {
        sendMotorControl(maxSpeed, 1, maxSpeed, 1);
      } else if (action === "left") {
        sendMotorControl(maxSpeed, 1, maxSpeed, 0);
      } else if (action === "right") {
        sendMotorControl(maxSpeed, 0, maxSpeed, 1);
      } else if (action === "stop") {
        sendMotorControl(0, 0, 0, 0);
      }
      
      // Update UI status for button control
      updateButtonStatusUI(action);
    }
    
    // Update status for button controls
    function updateButtonStatusUI(action) {
      statusIndicator.classList.remove("status-stopped", "status-moving");
      
      if (action === "stop") {
        statusIndicator.classList.add("status-stopped");
        statusText.innerText = "Motors Stopped";
      } else {
        statusIndicator.classList.add("status-moving");
        if (action === "forward") statusText.innerText = "Forward";
        else if (action === "reverse") statusText.innerText = "Backward";
        else if (action === "left") statusText.innerText = "Turning Left";
        else if (action === "right") statusText.innerText = "Turning Right";
      }
    }
    
    // Send motor control command to the server
    function sendMotorControl(leftSpeed, leftDir, rightSpeed, rightDir) {
      var xhr = new XMLHttpRequest();
      xhr.open("GET", `/motorJoy?la=${leftSpeed}&ld=${leftDir}&ra=${rightSpeed}&rd=${rightDir}`, true);
      xhr.send();
    }
    
    // Update maximum speed setting
    function updateMaxSpeed(value) {
      document.getElementById("speedDisplay").innerHTML = value;
      maxSpeed = parseInt(value);
      
      var xhr = new XMLHttpRequest();
      xhr.open("GET", "/speed?value=" + value, true);
      xhr.send();
    }
    
    // Switch between joystick and button control modes
    function switchMode(mode) {
      const joystickControls = document.getElementById('joystick-controls');
      const buttonControls = document.getElementById('button-controls');
      const joystickModeBtn = document.getElementById('joystick-mode');
      const buttonsModeBtn = document.getElementById('buttons-mode');
      
      if (mode === 'joystick') {
        joystickControls.classList.remove('hidden');
        buttonControls.classList.add('hidden');
        joystickModeBtn.classList.add('mode-active');
        joystickModeBtn.classList.remove('mode-inactive');
        buttonsModeBtn.classList.add('mode-inactive');
        buttonsModeBtn.classList.remove('mode-active');
      } else {
        joystickControls.classList.add('hidden');
        buttonControls.classList.remove('hidden');
        joystickModeBtn.classList.add('mode-inactive');
        joystickModeBtn.classList.remove('mode-active');
        buttonsModeBtn.classList.add('mode-active');
        buttonsModeBtn.classList.remove('mode-inactive');
      }
      
      // Stop motors when switching modes
      sendMotorControl(0, 0, 0, 0);
      statusIndicator.classList.remove("status-moving");
      statusIndicator.classList.add("status-stopped");
      statusText.innerText = "Motors Stopped";
    }
    
    // Initialize everything when page loads
    window.onload = function() {
      initJoystick();
    };
  </script>
</body>
</html>
)rawliteral";

void setup() {
  // Initialize serial communication
  Serial.begin(115200);
  Serial.println();
  
  // Set motor control pins as outputs
  pinMode(pwmMotorA, OUTPUT);
  pinMode(pwmMotorB, OUTPUT);
  pinMode(dirMotorA, OUTPUT);
  pinMode(dirMotorB, OUTPUT);
  
  // Initially stop motors
  digitalWrite(pwmMotorA, 0);
  digitalWrite(pwmMotorB, 0);
  
  // Create Access Point
  WiFi.softAP(ssid, password);
  
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(myIP);
  
  // Set up web server routes
  server.on("/", handleRoot);
  server.on("/motor", handleMotor);
  server.on("/motorJoy", handleJoystick);
  server.on("/speed", handleSpeed);
  
  // Start the server
  server.begin();
  Serial.println("HTTP server started");
  Serial.println("Robot Control Center Initialized");
  Serial.print("Connect to Wi-Fi network: ");
  Serial.println(ssid);
  Serial.print("Password: ");
  Serial.println(password);
  Serial.print("Then navigate to http://");
  Serial.println(myIP);
}

void loop() {
  server.handleClient();
}

void handleRoot() {
  server.send(200, "text/html", htmlPage);
}

void handleMotor() {
  String action = server.arg("action");
  
  if (action == "forward") {
    // Both motors forward
    digitalWrite(pwmMotorA, motorSpeed);
    digitalWrite(dirMotorA, LOW);
    digitalWrite(pwmMotorB, motorSpeed);
    digitalWrite(dirMotorB, LOW);
    Serial.println("Moving Forward");
  } 
  else if (action == "reverse") {
    // Both motors reverse
    digitalWrite(pwmMotorA, motorSpeed);
    digitalWrite(dirMotorA, HIGH);
    digitalWrite(pwmMotorB, motorSpeed);
    digitalWrite(dirMotorB, HIGH);
    Serial.println("Moving Backward");
  }
  else if (action == "left") {
    // Left motor reverse, right motor forward
    digitalWrite(pwmMotorA, motorSpeed);
    digitalWrite(dirMotorA, HIGH);  // Left motor reverse
    digitalWrite(pwmMotorB, motorSpeed);
    digitalWrite(dirMotorB, LOW);   // Right motor forward
    Serial.println("Turning Left");
  }
  else if (action == "right") {
    // Left motor forward, right motor reverse
    digitalWrite(pwmMotorA, motorSpeed);
    digitalWrite(dirMotorA, LOW);   // Left motor forward
    digitalWrite(pwmMotorB, motorSpeed);
    digitalWrite(dirMotorB, HIGH);  // Right motor reverse
    Serial.println("Turning Right");
  }
  else if (action == "stop") {
    // Stop both motors
    digitalWrite(pwmMotorA, 0);
    digitalWrite(pwmMotorB, 0);
    Serial.println("Motors Stopped");
  }
  
  server.send(200, "text/plain", "OK");
}

void handleJoystick() {
  // Get parameters from joystick
  int leftSpeed = server.arg("la").toInt();
  int leftDir = server.arg("ld").toInt();
  int rightSpeed = server.arg("ra").toInt();
  int rightDir = server.arg("rd").toInt();
  
  // Set motor directions (HIGH or LOW)
  bool dirA = leftDir == 1 ? HIGH : LOW;
  bool dirB = rightDir == 1 ? HIGH : LOW;
  
  // Set motor speeds
  digitalWrite(dirMotorA, dirA);
  digitalWrite(dirMotorB, dirB);
  
  // Apply speeds to motors
  analogWrite(pwmMotorA, leftSpeed);
  analogWrite(pwmMotorB, rightSpeed);
  
  // Debug output
  Serial.print("Left: ");
  Serial.print(leftSpeed);
  Serial.print(leftDir == 1 ? " REV, " : " FWD, ");
  Serial.print("Right: ");
  Serial.print(rightSpeed);
  Serial.println(rightDir == 1 ? " REV" : " FWD");
  
  server.send(200, "text/plain", "OK");
}

void handleSpeed() {
  int value = server.arg("value").toInt();
  
  // Update the speed variable
  motorSpeed = value;
  
  // Update the active motors immediately if using button interface
  if (digitalRead(pwmMotorA) != 0) {
    digitalWrite(pwmMotorA, motorSpeed);
  }
  if (digitalRead(pwmMotorB) != 0) {
    digitalWrite(pwmMotorB, motorSpeed);
  }
  
  Serial.print("Max Motor Speed Set to: ");
  Serial.println(motorSpeed);
  
  server.send(200, "text/plain", "OK");
}